@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2014 Christoph Berg <cb@df7cb.de>
+# Copyright (C) 2004 Christoph Berg <cb@df7cb.de>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -9,6 +9,13 @@
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# 2004-07-04 cb: added links and cleaned up code
+# 2004-07-24 cb: added viewport
 
 package IrcMarkers::File;
 
@@ -22,20 +29,9 @@ sub despace {
 	return $s;
 }
 
-sub parse_coord {
-	$_ = shift;
-	s/^[NE+]\s*//;
-	s/^[SW-]\s*/-/;
-	s/[°'"]+/ /g; # noise
-	s/,/./g;
-	s/(\d+) +([\d.]+)/$1 + $2 \/ 60.0/e;
-	s/([\d.]+) +([\d.]+)/$1 + $2 \/ 3600.0/e;
-	return $_;
-}
-
 sub new {
 	my $config = { # default values
-		projection => 'square',
+		projection => 'mercator',
 		west => -180,
 		north => 90,
 		east => 180,
@@ -56,7 +52,6 @@ sub new {
 		quiet => 0,
 		overlap => '/usr/lib/ircmarkers/overlap',
 		overlap_correction => 1,
-		help_convert_crop => 0,
 	};
 	bless $config;
 }
@@ -75,29 +70,22 @@ sub default_options { # this will be obsolete if all items are deOOified
 	$item->{link_color} ||= $config->{link_color}; # only for links
 }
 
-sub parser_warn {
-	my $config = shift;
-	my $text = shift;
-	my $loc = $config->{file} ? "$config->{file}:$." : "-o";
-	warn "$loc: $text\n";
-}
-
 sub parse_options {
 	my $config = shift;
 	my $item = shift;
 	die "huh?" unless ref $item;
-	my $markernr = shift; # defined for markers, undef for yxlabels/links
+	my $marker = shift; # defined for markers, undef for yxlabels/links
 	my $opt = shift;
 	while($opt ne '') { # loop over options
 		$opt =~ s/^\s+|#.*//g;
 		last unless $opt ne '';
-		if(defined $markernr and $opt =~ s/^gpg[ :](?:0x)?([0-9a-f]{16})//i) { # ':' is deprecated old syntax
+		if($marker and $opt =~ s/^gpg[ :](?:0x)?([0-9a-f]{16})//i) { # ':' is deprecated old syntax
 			my $k = uc $1;
-			if($config->{gpg}->{$k} and $config->{gpg}->{$k} ne $markernr) {
-				warn "$config->{file}.$.: key $k already associated with $config->{gpg}->{$k}, overwriting with $markernr\n";
+			if($config->{gpg}->{$k} and $config->{gpg}->{$k} ne $marker) {
+				warn "$config->{file}.$.: key $k already associated with $config->{gpg}->{$k}, overwriting with $marker\n";
 			}
-			$config->{gpg}->{$k} = $markernr;
-			$config->{gpg_not_found}->{$k} = $markernr;
+			$config->{gpg}->{$k} = $marker;
+			$config->{gpg_not_found}->{$k} = $marker;
 		# local options
 		} elsif($opt =~ s/^dot_colou?r (\d+) (\d+) (\d+)//) {
 			$item->{dot_color} = [$1, $2, $3];
@@ -119,40 +107,33 @@ sub parse_options {
 			die "font file not found: $1" unless -f $1;
 			$item->{font} = $1;
 			$item->{font} =~ s!^~/!$ENV{HOME}/!;
-		} elsif($opt =~ s/^(?:font|pt)size (\d+)//) {
+		} elsif($opt =~ s/^ptsize (\d+)//) {
 			$item->{ptsize} = $1;
 		} elsif($opt =~ s/^(?:link|sign2)_colou?r (\d+) (\d+) (\d+)//) {
 			$item->{link_color} = [$1, $2, $3];
 		} elsif($opt =~ s/^(?:link|sign2)_colou?r (no(ne?)|off)//) {
 			delete $item->{link_color};
-		# (mostly) pisg options
-		} elsif($opt =~ s/^alias "(.*?)"//) {
-			$item->{alias} = $item->{alias} ? "$item->{alias} $1" : $1;
-		} elsif($opt =~ s/^alias (\S+)//) {
-			$item->{alias} = $item->{alias} ? "$item->{alias} $1" : $1;
-		} elsif($opt =~ s/^(href|(?:big)?pic|sex) ("?)(\S+)\2//) {
-			$item->{$1} = $3;
 		# error
 		} else {
-			$config->parser_warn("unknown option: $opt");
+			my $loc = $config->{file} ? "$config->{file}:$." : "-o";
+			warn "$loc: unknown option: $opt\n";
 			last;
 		}
 	}
 }
 
-my $markernr = 0;
 my $labelnr = 0;
 my $linknr = 0;
 sub parse {
 	my $config = shift;
 	$_ = shift;
 
-	s/^\s+//;
+	return if /^\s*$/;
 	if(/^#include [<"](.+)[">]/) { # include next config file
 		$config->read($1);
 		return;
 	}
-	return if /^(#|$)/;
+	return if /^\s*#/;
 
 	s/^([^"]+)/despace($1)/e; # compress space outside of quotes
 	s/([^"]+)$/despace($1)/e;
@@ -180,7 +161,7 @@ sub parse {
 		$config->{view_width} = $1;
 	} elsif(/^view_height (.+)/) {
 		$config->{view_height} = $1;
-	} elsif(/^projection (square|mercator|sinusoidal)/) {
+	} elsif(/^projection (mercator|sinusoidal)/) {
 		$config->{projection} = $1;
 	} elsif(/^center_lon (.+)/) {
 		$config->{center_lon} = $1;
@@ -192,79 +173,38 @@ sub parse {
 		delete $config->{sign1_color};
 	} elsif(/^sign1_colou?r (\d+) (\d+) (\d+)$/) {
 		$config->{sign1_color} = [$1, $2, $3];
-	} elsif(/^imagemap (\S+)/) {
-		$config->{imagemap} = $1;
 	} elsif(/^overlap (.+)/) {
 		$config->{overlap} = $1;
 	} elsif(/^overlap_correction (on|yes)/) {
 		$config->{overlap_correction} = 1;
 	} elsif(/^overlap_correction (off|no)/) {
 		$config->{overlap_correction} = 0;
-	} elsif(/^pisg/) {
-		$config->{pisg} = 1;
-	} elsif(/^recv(?:-keys)/) {
-		$config->{recv} = 1;
 	} elsif(/^quiet (on|yes)/) {
 		$config->{quiet} = 1;
 	} elsif(/^quiet (off|no)/) {
 		$config->{quiet} = 0;
 	# link
 	} elsif(/"([^"]*)"\s+<?->\s+"([^"]+)"(.*)/) { # -> is old syntax
-		my ($src, $dst, $opt) = ($1, $2, $3);
-		my $srcnr = $config->{markerindex}->{$src};
-		my $dstnr = $config->{markerindex}->{$dst};
-		unless(defined $srcnr and defined $dstnr) {
-			$config->parser_warn("both link endpoints must be defined");
-			next;
-		}
-		$config->{links}->[$linknr] = { src => $srcnr, dst => $dstnr, arrow => '<->' };
+		$config->{links}->[$linknr] = { src => $1, dst => $2 };
+		my $opt = $3;
 		$config->default_options($config->{links}->[$linknr]);
 		$config->parse_options($config->{links}->[$linknr], undef, $opt);
 		$linknr++;
 	# marker definitions
-	} elsif(/^([NS+-]? ?[\d.,°]+) ([EW+-]? ?[\d.,°]+) "([^"]*)"(.*)/ or
-		# N 51° 11.123 E 006° 25.846
-		/^([NS+-]? ?\d*[° ]+[\d.,]+'?) ([EW+-]? ?\d*[° ]+[\d.,]+'?) "([^"]*)"(.*)/ or
-		# N 51° 34' 11.123 E 006° 29' 25.846
-		/^([NS+-]? ?\d*[° ]+\d+[' ][\d.,]+"?) ([EW+-]? ?\d*[° ]+\d+[' ][\d.,]+"?) "([^"]*)"(.*)/) {
-		my ($lat, $lon, $text, $opt) = (parse_coord($1), parse_coord($2), $3, $4);
-		my $nr = $config->{markerindex}->{$text};
-		if(not defined $nr or $config->{markers}->[$nr]->{lat}) { # create new marker when coordinates are already there
-			$nr = $markernr++;
-		}
-		$config->{markers}->[$nr]->{text} = $text;
-		$config->{markers}->[$nr]->{lat} = $lat;
-		$config->{markers}->[$nr]->{lon} = $lon;
-		$config->{markerindex}->{$text} = $nr;
-		$config->default_options($config->{markers}->[$nr]);
-		$config->parse_options($config->{markers}->[$nr], $nr, $opt);
-	} elsif(/^([A-Z]{2}\d{2}[A-Z]{2}) "([^"]*)"(.*)/i) { # Maidenhead locator
-		my ($loc, $text, $opt) = ($1, $2, $3);
-		my ($p1, $p2, $p3, $p4, $p5, $p6) = unpack 'AAAAAA', uc($loc);
-		($p1, $p2, $p3, $p4, $p5, $p6) = (ord($p1)-ord('A'), ord($p2)-ord('A'), ord($p3)-ord('0'), ord($p4)-ord('0'), ord($p5)-ord('A'), ord($p6)-ord('A') );
-		my $lat = ($p2*10) + $p4 + (($p6+0.5)/24) - 90;
-		my $lon = ($p1*20) + ($p3*2) + (($p5+0.5)/12) - 180;
-		my $nr = $config->{markerindex}->{$text};
-		if(not defined $nr or $config->{markers}->[$nr]->{lat}) { # create new marker when coordinates are already there
-			$nr = $markernr++;
-		}
-		$config->{markers}->[$nr]->{text} = $text;
-		$config->{markers}->[$nr]->{lat} = $lat;
-		$config->{markers}->[$nr]->{lon} = $lon;
-		$config->{markerindex}->{$text} = $nr;
-		$config->default_options($config->{markers}->[$nr]);
-		$config->parse_options($config->{markers}->[$nr], $nr, $opt);
+	} elsif(/^([\d.,-]+) ([\d.,-]+) "([^"]+)"(.*)/) { # xplanet marker file format
+		my ($lat, $lon, $marker, $opt) = ($1, $2, $3, $4);
+		$lat =~ s/,/./;
+		$lon =~ s/,/./;
+		$config->{markers}->{$marker}->{lat} = $lat;
+		$config->{markers}->{$marker}->{lon} = $lon;
+		$config->default_options($config->{markers}->{$marker});
+		$config->parse_options($config->{markers}->{$marker}, $marker, $opt);
 	} elsif(/^"([^"]*)"(.*)/) { # marker with options
-		my ($text, $opt, $i) = ($1, $2);
-		my $nr = $config->{markerindex}->{$text};
-		if(not defined $nr) {
-			$nr = $markernr++;
-			$config->{markers}->[$nr]->{text} = $text;
-		}
-		$config->default_options($config->{markers}->[$nr]);
-		$config->parse_options($config->{markers}->[$nr], $config->{markerindex}->{$text}, $opt);
+		my ($marker, $opt) = ($1, $2);
+		$config->{markers}->{$marker} ||= {};
+		$config->parse_options($config->{markers}->{$marker}, $marker, $opt);
 	} elsif(/^label ([+-]?\d+) ([+-]?\d+) "([^"]+)"(.*)/) {
-		$config->{yxlabels}->[$labelnr] = { labely => $1, labelx => $2, text => $3 };
+		$config->{yxlabels}->[$labelnr] = { y => $1, x => $2, text => $3 };
 		my $opt = $4;
 		$config->default_options($config->{yxlabels}->[$labelnr]);
 		$config->parse_options($config->{yxlabels}->[$labelnr], undef, $opt);
@@ -297,24 +237,21 @@ sub get_gpg_links {
 	my $config = shift;
 
 	my $keys = join ' ', keys %{$config->{gpg}};
-	if ($config->{recv}) {
-		system "gpg --recv-keys $keys";
-	}
-
+	#print "gpg --list-sigs --with-colon --fixed-list-mode --fast-list-mode $keys\n";
 	open GPG, "gpg --list-sigs --with-colon --fixed-list-mode --fast-list-mode $keys |" or die "gpg: $!";
-	my ($key, $src);
+	my $key;
 	while(<GPG>) {
 		chomp;
 		next if /^(rev|sub|tru|uat):/;
+		#print "$_\n";
 		if(/^pub::\d+:\d+:([0-9A-F]+):/) {
 			$key = $1;
-			warn "$key not related to any marker - did you use the long (16 char) keyid?\n" unless defined $config->{gpg}->{$key};
-			$src = $config->{gpg}->{$key};
+			warn "$key not related to any marker - did you use the long (16 char) keyid?\n" unless $config->{gpg}->{$key};
 			delete $config->{gpg_not_found}->{$key};
 		} elsif(/^sig:::\d+:([0-9A-F]+):/) {
-			next unless defined $config->{gpg}->{$1}; # target not on map
+			next unless $config->{gpg}->{$1}; # target not on map
 			next if $key eq $1; # self-sig
-			$config->{gpg_links}->{$config->{gpg}->{$1}}->{$src} = 1;
+			$config->{gpg_links}->{$1}->{$key} = 1;
 		} else {
 			warn "unknown gpg output: $_";
 		}
@@ -324,23 +261,33 @@ sub get_gpg_links {
 		warn "$config->{gpg_not_found}->{$key}: key $key was not found in gpg's keyring\n";
 	}
 
-	foreach my $source (keys %{$config->{gpg_links}}) {
-		next unless $config->{link_outside} or $config->{markers}->[$source]->{visible};
-		foreach my $target (keys %{$config->{gpg_links}->{$source}}) {
-			next unless $config->{link_outside} or $config->{markers}->[$target]->{visible};
+	foreach $key (keys %{$config->{gpg}}) {
+		my $marker = $config->{gpg}->{$key};
+		unless($config->{markers}->{$marker}->{lat}) {
+			warn "$marker was mentioned as key $key, but no coordinates defined\n";
+		}
+	}
+
+	foreach my $key (keys %{$config->{gpg_links}}) {
+		my $source = $config->{gpg}->{$key};
+		next unless $config->{link_outside} or $config->{markers}->{$source}->{visible};
+		foreach my $targetkey (keys %{$config->{gpg_links}->{$key}}) {
+			my $target = $config->{gpg}->{$targetkey};
+			next unless $config->{link_outside} or $config->{markers}->{$target}->{visible};
 			my ($arrow, $color);
-			if($config->{gpg_links}->{$target}->{$source}) { # bidirectional link
-				next if $target gt $source; # process only once
+			# TODO: compute uni/bidi for markers instead of keys (for people with more than one key)
+			if($config->{gpg_links}->{$targetkey}->{$key}) { # bidirectional link
+				next if $targetkey gt $key; # process only once
 				$color = $config->{link_color};
 				$arrow = "<->";
 			} else {
 				$color = $config->{sign1_color} or next; # don't draw unidirectional links
 				$arrow = "-->";
 			}
-			next if not defined $config->{markers}->[$source]->{lat};
-			next if not defined $config->{markers}->[$target]->{lat};
-			push @{ $config->{links} },
-				{ src => $source, dst => $target, link_color => $color, arrow => $arrow };
+			next unless $config->{markers}->{$source}->{lat};
+			next unless $config->{markers}->{$target}->{lat};
+			$config->link($source, $target, $color);
+			print "$source $arrow $target\n" unless $config->{quiet};
 		}
 	}
 }
